'''OpenGL extension DFX.tbuffer

Overview (from the spec)
	
	This extension allows a write mask to be defined for the fragment
	mask which is created during multisample rendering. This can be used
	to create effects such as motion blur and depth of field.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/DFX/tbuffer.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_DFX_tbuffer'

glTbufferMask3DFX = platform.createExtensionFunction( 
	'glTbufferMask3DFX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glTbufferMask3DFX( GLuint(mask) ) -> None',
	argNames = ('mask',),
)


def glInitTbufferDFX():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""GLE exceptional functions (specialised signatures"""
from OpenGL.raw import GLE as raw
from OpenGL.raw.GLE import annotations as simple

from OpenGL import wrapper, arrays

class _lengthOfArgname( object ):
	"""Calculates the length of a given argname over a divisor value"""
	def __init__( self, arrayName, divisor, arrayType = arrays.GLdoubleArray ):
		self.arrayName = arrayName
		self.divisor = divisor
		self.arrayType = arrayType
	def finalise( self, wrapper ):
		self.arrayIndex = wrapper.pyArgIndex( self.arrayName )
	def __call__( self, pyArgs, index, wrappedOperation ):
		"""Get the length of pyArgs[2], a glDoubleArray"""
		return self.arrayType.arraySize( pyArgs[self.arrayIndex] )//self.divisor
def _baseWrap( base, lengthName='ncp', contourName='contour', divisor=2 ):
	"""Do the basic wrapping operation for a GLE function"""
	return wrapper.wrapper( base ).setPyConverter(
		lengthName,
	).setCConverter(
		lengthName, _lengthOfArgname( contourName, divisor, arrays.GLdoubleArray ),
	)

gleLathe = _baseWrap( simple.gleLathe )
glePolyCone = _baseWrap( simple.glePolyCone, 'npoints', 'point_array', 3)
glePolyCylinder = _baseWrap( simple.glePolyCylinder, 'npoints', 'point_array', 3)
gleScrew = _baseWrap( simple.gleScrew )
gleSpiral = _baseWrap( simple.gleSpiral )

gleExtrusion = _baseWrap( 
	_baseWrap( simple.gleExtrusion ),
	'npoints', 'point_array', 3
)
gleSuperExtrusion = _baseWrap( 
	_baseWrap( simple.gleSuperExtrusion ),
	'npoints', 'point_array', 3
)
gleTwistExtrusion = _baseWrap( 
	_baseWrap( simple.gleTwistExtrusion ),
	'npoints', 'point_array', 3
)



'''OpenGL extension ATI.envmap_bumpmap

Overview (from the spec)
	
	This extension adds environment mapped bump mapping (EMBM) to the GL.
	The method exposed by this extension is to use a dependent texture
	read on a bumpmap (du,dv) texture to offset the texture coordinates
	read into a map on another texture unit.  This (du,dv) offset is also 
	rotated through a user-specified rotation matrix to get the texture 
	coordinates into the appropriate space.
	
	A new texture format is introduced in order for specifying the (du,dv)
	bumpmap texture.  This map represents -1 <= du,dv <= 1 offsets to
	be applied to the texture coordinates used to read into the base
	map.  Additionally, the (du,dv) offsets are transformed by a rotation
	matrix that this extension allows the user to specify.  Further, a 
	new color operation is added to EXT_texture_env_combine to specify 
	both that bumpmapping is enabled and which texture unit to apply 
	the bump offset to.  

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ATI/envmap_bumpmap.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ATI_envmap_bumpmap'
GL_BUMP_ROT_MATRIX_ATI = constant.Constant( 'GL_BUMP_ROT_MATRIX_ATI', 0x8775 )
GL_BUMP_ROT_MATRIX_SIZE_ATI = constant.Constant( 'GL_BUMP_ROT_MATRIX_SIZE_ATI', 0x8776 )
GL_BUMP_NUM_TEX_UNITS_ATI = constant.Constant( 'GL_BUMP_NUM_TEX_UNITS_ATI', 0x8777 )
GL_BUMP_TEX_UNITS_ATI = constant.Constant( 'GL_BUMP_TEX_UNITS_ATI', 0x8778 )
GL_DUDV_ATI = constant.Constant( 'GL_DUDV_ATI', 0x8779 )
GL_DU8DV8_ATI = constant.Constant( 'GL_DU8DV8_ATI', 0x877A )
GL_BUMP_ENVMAP_ATI = constant.Constant( 'GL_BUMP_ENVMAP_ATI', 0x877B )
GL_BUMP_TARGET_ATI = constant.Constant( 'GL_BUMP_TARGET_ATI', 0x877C )
glTexBumpParameterivATI = platform.createExtensionFunction( 
	'glTexBumpParameterivATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glTexBumpParameterivATI( GLenum(pname), GLintArray(param) ) -> None',
	argNames = ('pname', 'param',),
)

glTexBumpParameterfvATI = platform.createExtensionFunction( 
	'glTexBumpParameterfvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glTexBumpParameterfvATI( GLenum(pname), GLfloatArray(param) ) -> None',
	argNames = ('pname', 'param',),
)

glGetTexBumpParameterivATI = platform.createExtensionFunction( 
	'glGetTexBumpParameterivATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glGetTexBumpParameterivATI( GLenum(pname), GLintArray(param) ) -> None',
	argNames = ('pname', 'param',),
)

glGetTexBumpParameterfvATI = platform.createExtensionFunction( 
	'glGetTexBumpParameterfvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glGetTexBumpParameterfvATI( GLenum(pname), GLfloatArray(param) ) -> None',
	argNames = ('pname', 'param',),
)


def glInitEnvmapBumpmapATI():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.point_sprite

Overview (from the spec)
	
	Applications such as particle systems usually must use OpenGL quads
	rather than points to render their geometry, since they would like to
	use a custom-drawn texture for each particle, rather than the
	traditional OpenGL round antialiased points, and each fragment in
	a point has the same texture coordinates as every other fragment.
	
	Unfortunately, specifying the geometry for these quads can be quite
	expensive, since it quadruples the amount of geometry required, and
	it may also require the application to do extra processing to compute
	the location of each vertex.
	
	The goal of this extension is to allow such apps to use points rather
	than quads.  When GL_POINT_SPRITE_NV is enabled, the state of point
	antialiasing is ignored.  For each texture unit, the app can then
	specify whether to replace the existing texture coordinates with
	point sprite texture coordinates, which are interpolated across the
	point.  Finally, the app can set a global parameter for the way to
	generate the R coordinate for point sprites; the R coordinate can
	either be zero, the input S coordinate, or the input R coordinate.
	This allows applications to use a 3D texture to represent a point
	sprite that goes through an animation, with filtering between frames,
	for example.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/point_sprite.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_point_sprite'
GL_POINT_SPRITE_NV = constant.Constant( 'GL_POINT_SPRITE_NV', 0x8861 )
glget.addGLGetConstant( GL_POINT_SPRITE_NV, (1,) )
GL_COORD_REPLACE_NV = constant.Constant( 'GL_COORD_REPLACE_NV', 0x8862 )
GL_POINT_SPRITE_R_MODE_NV = constant.Constant( 'GL_POINT_SPRITE_R_MODE_NV', 0x8863 )
glget.addGLGetConstant( GL_POINT_SPRITE_R_MODE_NV, (1,) )
glPointParameteriNV = platform.createExtensionFunction( 
	'glPointParameteriNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint,),
	doc = 'glPointParameteriNV( GLenum(pname), GLint(param) ) -> None',
	argNames = ('pname', 'param',),
)

glPointParameterivNV = platform.createExtensionFunction( 
	'glPointParameterivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glPointParameterivNV( GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('pname', 'params',),
)


def glInitPointSpriteNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



#!/usr/bin/python

import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *

def main():
	pygame.init()
	pygame.display.set_mode((800,600), pygame.OPENGL|
		pygame.DOUBLEBUF)
	glClearColor(1.0, 0.0, 0.0, 1.0)
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)

	done = False

	while not done:
		for i in range(0,50000):
			modelview_matrix = glGetFloatv(GL_MODELVIEW_MATRIX)
			if not i % 500:
				print '.',
		print
		pygame.display.flip()

		eventlist = pygame.event.get()
		for event in eventlist:
			if event.type == QUIT or event.type == KEYDOWN and event.key == K_ESCAPE:
				done = True

if __name__ == '__main__':
	main()



'''OpenGL extension ARB.shader_objects

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/shader_objects.txt
'''
### AUTOGENERATED DO NOT EDIT
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, finalisewrappers, wrapper
import ctypes
GL_PROGRAM_OBJECT_ARB = constant.Constant( 'GL_PROGRAM_OBJECT_ARB', 0x8B40 )
GL_SHADER_OBJECT_ARB = constant.Constant( 'GL_SHADER_OBJECT_ARB', 0x8B48 )
GL_OBJECT_TYPE_ARB = constant.Constant( 'GL_OBJECT_TYPE_ARB', 0x8B4E )
GL_OBJECT_SUBTYPE_ARB = constant.Constant( 'GL_OBJECT_SUBTYPE_ARB', 0x8B4F )
GL_FLOAT_VEC2_ARB = constant.Constant( 'GL_FLOAT_VEC2_ARB', 0x8B50 )
GL_FLOAT_VEC3_ARB = constant.Constant( 'GL_FLOAT_VEC3_ARB', 0x8B51 )
GL_FLOAT_VEC4_ARB = constant.Constant( 'GL_FLOAT_VEC4_ARB', 0x8B52 )
GL_INT_VEC2_ARB = constant.Constant( 'GL_INT_VEC2_ARB', 0x8B53 )
GL_INT_VEC3_ARB = constant.Constant( 'GL_INT_VEC3_ARB', 0x8B54 )
GL_INT_VEC4_ARB = constant.Constant( 'GL_INT_VEC4_ARB', 0x8B55 )
GL_BOOL_ARB = constant.Constant( 'GL_BOOL_ARB', 0x8B56 )
GL_BOOL_VEC2_ARB = constant.Constant( 'GL_BOOL_VEC2_ARB', 0x8B57 )
GL_BOOL_VEC3_ARB = constant.Constant( 'GL_BOOL_VEC3_ARB', 0x8B58 )
GL_BOOL_VEC4_ARB = constant.Constant( 'GL_BOOL_VEC4_ARB', 0x8B59 )
GL_FLOAT_MAT2_ARB = constant.Constant( 'GL_FLOAT_MAT2_ARB', 0x8B5A )
GL_FLOAT_MAT3_ARB = constant.Constant( 'GL_FLOAT_MAT3_ARB', 0x8B5B )
GL_FLOAT_MAT4_ARB = constant.Constant( 'GL_FLOAT_MAT4_ARB', 0x8B5C )
GL_SAMPLER_1D_ARB = constant.Constant( 'GL_SAMPLER_1D_ARB', 0x8B5D )
GL_SAMPLER_2D_ARB = constant.Constant( 'GL_SAMPLER_2D_ARB', 0x8B5E )
GL_SAMPLER_3D_ARB = constant.Constant( 'GL_SAMPLER_3D_ARB', 0x8B5F )
GL_SAMPLER_CUBE_ARB = constant.Constant( 'GL_SAMPLER_CUBE_ARB', 0x8B60 )
GL_SAMPLER_1D_SHADOW_ARB = constant.Constant( 'GL_SAMPLER_1D_SHADOW_ARB', 0x8B61 )
GL_SAMPLER_2D_SHADOW_ARB = constant.Constant( 'GL_SAMPLER_2D_SHADOW_ARB', 0x8B62 )
GL_SAMPLER_2D_RECT_ARB = constant.Constant( 'GL_SAMPLER_2D_RECT_ARB', 0x8B63 )
GL_SAMPLER_2D_RECT_SHADOW_ARB = constant.Constant( 'GL_SAMPLER_2D_RECT_SHADOW_ARB', 0x8B64 )
GL_OBJECT_DELETE_STATUS_ARB = constant.Constant( 'GL_OBJECT_DELETE_STATUS_ARB', 0x8B80 )
GL_OBJECT_COMPILE_STATUS_ARB = constant.Constant( 'GL_OBJECT_COMPILE_STATUS_ARB', 0x8B81 )
GL_OBJECT_LINK_STATUS_ARB = constant.Constant( 'GL_OBJECT_LINK_STATUS_ARB', 0x8B82 )
GL_OBJECT_VALIDATE_STATUS_ARB = constant.Constant( 'GL_OBJECT_VALIDATE_STATUS_ARB', 0x8B83 )
GL_OBJECT_INFO_LOG_LENGTH_ARB = constant.Constant( 'GL_OBJECT_INFO_LOG_LENGTH_ARB', 0x8B84 )
GL_OBJECT_ATTACHED_OBJECTS_ARB = constant.Constant( 'GL_OBJECT_ATTACHED_OBJECTS_ARB', 0x8B85 )
GL_OBJECT_ACTIVE_UNIFORMS_ARB = constant.Constant( 'GL_OBJECT_ACTIVE_UNIFORMS_ARB', 0x8B86 )
GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB = constant.Constant( 'GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB', 0x8B87 )
GL_OBJECT_SHADER_SOURCE_LENGTH_ARB = constant.Constant( 'GL_OBJECT_SHADER_SOURCE_LENGTH_ARB', 0x8B88 )
glDeleteObjectARB = platform.createExtensionFunction( 
	'glDeleteObjectARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB,),
	doc = 'glDeleteObjectARB( GLhandleARB(obj) ) -> None',
	argNames = ('obj',),
)

glGetHandleARB = platform.createExtensionFunction( 
	'glGetHandleARB', dll=platform.GL,
	resultType=constants.GLhandleARB, 
	argTypes=(constants.GLenum,),
	doc = 'glGetHandleARB( GLenum(pname) ) -> constants.GLhandleARB',
	argNames = ('pname',),
)

glDetachObjectARB = platform.createExtensionFunction( 
	'glDetachObjectARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLhandleARB,),
	doc = 'glDetachObjectARB( GLhandleARB(containerObj), GLhandleARB(attachedObj) ) -> None',
	argNames = ('containerObj', 'attachedObj',),
)

glCreateShaderObjectARB = platform.createExtensionFunction( 
	'glCreateShaderObjectARB', dll=platform.GL,
	resultType=constants.GLhandleARB, 
	argTypes=(constants.GLenum,),
	doc = 'glCreateShaderObjectARB( GLenum(shaderType) ) -> constants.GLhandleARB',
	argNames = ('shaderType',),
)

glShaderSourceARB = platform.createExtensionFunction( 
	'glShaderSourceARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLsizei, ctypes.POINTER(arrays.GLcharARBArray), arrays.GLintArray,),
	doc = 'glShaderSourceARB( GLhandleARB(shaderObj), GLsizei(count), POINTER(arrays.GLcharARBArray)(string), GLintArray(length) ) -> None',
	argNames = ('shaderObj', 'count', 'string', 'length',),
)

glCompileShaderARB = platform.createExtensionFunction( 
	'glCompileShaderARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB,),
	doc = 'glCompileShaderARB( GLhandleARB(shaderObj) ) -> None',
	argNames = ('shaderObj',),
)

glCreateProgramObjectARB = platform.createExtensionFunction( 
	'glCreateProgramObjectARB', dll=platform.GL,
	resultType=constants.GLhandleARB, 
	argTypes=(),
	doc = 'glCreateProgramObjectARB(  ) -> constants.GLhandleARB',
	argNames = (),
)

glAttachObjectARB = platform.createExtensionFunction( 
	'glAttachObjectARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLhandleARB,),
	doc = 'glAttachObjectARB( GLhandleARB(containerObj), GLhandleARB(obj) ) -> None',
	argNames = ('containerObj', 'obj',),
)

glLinkProgramARB = platform.createExtensionFunction( 
	'glLinkProgramARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB,),
	doc = 'glLinkProgramARB( GLhandleARB(programObj) ) -> None',
	argNames = ('programObj',),
)

glUseProgramObjectARB = platform.createExtensionFunction( 
	'glUseProgramObjectARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB,),
	doc = 'glUseProgramObjectARB( GLhandleARB(programObj) ) -> None',
	argNames = ('programObj',),
)

glValidateProgramARB = platform.createExtensionFunction( 
	'glValidateProgramARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB,),
	doc = 'glValidateProgramARB( GLhandleARB(programObj) ) -> None',
	argNames = ('programObj',),
)

glUniform1fARB = platform.createExtensionFunction( 
	'glUniform1fARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLfloat,),
	doc = 'glUniform1fARB( GLint(location), GLfloat(v0) ) -> None',
	argNames = ('location', 'v0',),
)

glUniform2fARB = platform.createExtensionFunction( 
	'glUniform2fARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLfloat, constants.GLfloat,),
	doc = 'glUniform2fARB( GLint(location), GLfloat(v0), GLfloat(v1) ) -> None',
	argNames = ('location', 'v0', 'v1',),
)

glUniform3fARB = platform.createExtensionFunction( 
	'glUniform3fARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glUniform3fARB( GLint(location), GLfloat(v0), GLfloat(v1), GLfloat(v2) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2',),
)

glUniform4fARB = platform.createExtensionFunction( 
	'glUniform4fARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLfloat, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glUniform4fARB( GLint(location), GLfloat(v0), GLfloat(v1), GLfloat(v2), GLfloat(v3) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2', 'v3',),
)

glUniform1iARB = platform.createExtensionFunction( 
	'glUniform1iARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint,),
	doc = 'glUniform1iARB( GLint(location), GLint(v0) ) -> None',
	argNames = ('location', 'v0',),
)

glUniform2iARB = platform.createExtensionFunction( 
	'glUniform2iARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glUniform2iARB( GLint(location), GLint(v0), GLint(v1) ) -> None',
	argNames = ('location', 'v0', 'v1',),
)

glUniform3iARB = platform.createExtensionFunction( 
	'glUniform3iARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glUniform3iARB( GLint(location), GLint(v0), GLint(v1), GLint(v2) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2',),
)

glUniform4iARB = platform.createExtensionFunction( 
	'glUniform4iARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glUniform4iARB( GLint(location), GLint(v0), GLint(v1), GLint(v2), GLint(v3) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2', 'v3',),
)

glUniform1fvARB = platform.createExtensionFunction( 
	'glUniform1fvARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glUniform1fvARB( GLint(location), GLsizei(count), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform2fvARB = platform.createExtensionFunction( 
	'glUniform2fvARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glUniform2fvARB( GLint(location), GLsizei(count), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform3fvARB = platform.createExtensionFunction( 
	'glUniform3fvARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glUniform3fvARB( GLint(location), GLsizei(count), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform4fvARB = platform.createExtensionFunction( 
	'glUniform4fvARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glUniform4fvARB( GLint(location), GLsizei(count), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform1ivARB = platform.createExtensionFunction( 
	'glUniform1ivARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glUniform1ivARB( GLint(location), GLsizei(count), GLintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform2ivARB = platform.createExtensionFunction( 
	'glUniform2ivARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glUniform2ivARB( GLint(location), GLsizei(count), GLintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform3ivARB = platform.createExtensionFunction( 
	'glUniform3ivARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glUniform3ivARB( GLint(location), GLsizei(count), GLintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform4ivARB = platform.createExtensionFunction( 
	'glUniform4ivARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glUniform4ivARB( GLint(location), GLsizei(count), GLintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniformMatrix2fvARB = platform.createExtensionFunction( 
	'glUniformMatrix2fvARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix2fvARB( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glUniformMatrix3fvARB = platform.createExtensionFunction( 
	'glUniformMatrix3fvARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix3fvARB( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glUniformMatrix4fvARB = platform.createExtensionFunction( 
	'glUniformMatrix4fvARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix4fvARB( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glGetObjectParameterfvARB = platform.createExtensionFunction( 
	'glGetObjectParameterfvARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLenum, arrays.GLfloatArray,),
	doc = 'glGetObjectParameterfvARB( GLhandleARB(obj), GLenum(pname), GLfloatArray(params) ) -> None',
	argNames = ('obj', 'pname', 'params',),
)

glGetObjectParameterivARB = platform.createExtensionFunction( 
	'glGetObjectParameterivARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetObjectParameterivARB( GLhandleARB(obj), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('obj', 'pname', 'params',),
)

glGetInfoLogARB = platform.createExtensionFunction( 
	'glGetInfoLogARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLsizei, arrays.GLsizeiArray, arrays.GLcharARBArray,),
	doc = 'glGetInfoLogARB( GLhandleARB(obj), GLsizei(maxLength), GLsizeiArray(length), GLcharARBArray(infoLog) ) -> None',
	argNames = ('obj', 'maxLength', 'length', 'infoLog',),
)

glGetAttachedObjectsARB = platform.createExtensionFunction( 
	'glGetAttachedObjectsARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLsizei, arrays.GLsizeiArray, arrays.GLuintArray,),
	doc = 'glGetAttachedObjectsARB( GLhandleARB(containerObj), GLsizei(maxCount), GLsizeiArray(count), GLuintArray(obj) ) -> None',
	argNames = ('containerObj', 'maxCount', 'count', 'obj',),
)

glGetUniformLocationARB = platform.createExtensionFunction( 
	'glGetUniformLocationARB', dll=platform.GL,
	resultType=constants.GLint, 
	argTypes=(constants.GLhandleARB, arrays.GLcharARBArray,),
	doc = 'glGetUniformLocationARB( GLhandleARB(programObj), GLcharARBArray(name) ) -> constants.GLint',
	argNames = ('programObj', 'name',),
)

glGetActiveUniformARB = platform.createExtensionFunction( 
	'glGetActiveUniformARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLuint, constants.GLsizei, arrays.GLsizeiArray, arrays.GLintArray, arrays.GLuintArray, arrays.GLcharARBArray,),
	doc = 'glGetActiveUniformARB( GLhandleARB(programObj), GLuint(index), GLsizei(maxLength), GLsizeiArray(length), GLintArray(size), GLuintArray(type), GLcharARBArray(name) ) -> None',
	argNames = ('programObj', 'index', 'maxLength', 'length', 'size', 'type', 'name',),
)

glGetUniformfvARB = platform.createExtensionFunction( 
	'glGetUniformfvARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLint, arrays.GLfloatArray,),
	doc = 'glGetUniformfvARB( GLhandleARB(programObj), GLint(location), GLfloatArray(params) ) -> None',
	argNames = ('programObj', 'location', 'params',),
)

glGetUniformivARB = platform.createExtensionFunction( 
	'glGetUniformivARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLint, arrays.GLintArray,),
	doc = 'glGetUniformivARB( GLhandleARB(programObj), GLint(location), GLintArray(params) ) -> None',
	argNames = ('programObj', 'location', 'params',),
)

glGetShaderSourceARB = platform.createExtensionFunction( 
	'glGetShaderSourceARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLsizei, arrays.GLsizeiArray, arrays.GLcharARBArray,),
	doc = 'glGetShaderSourceARB( GLhandleARB(obj), GLsizei(maxLength), GLsizeiArray(length), GLcharARBArray(source) ) -> None',
	argNames = ('obj', 'maxLength', 'length', 'source',),
)


def glInitShaderObjectsARB():
	'''Return boolean indicating whether this module is available'''
	finalisewrappers.finalise( globals() )
	return extensions.hasGLExtension( 'GL_ARB_shader_objects' )
### END AUTOGENERATED SECTION
GL_INFO_LOG_LENGTH_ARB = constant.Constant( 'GL_INFO_LOG_LENGTH', 0x8B84 )

##glShaderSourceARB.argtypes = [c_int, c_int, POINTER(c_char_p), POINTER(c_int)]
##glGetObjectParameterivARB.argtypes = [c_int, c_int, POINTER(c_int)]
##glGetInfoLogARB.argtypes = [c_int, c_int, POINTER(c_int), c_char_p]

base_glShaderSourceARB = glShaderSourceARB
def glShaderSourceARB( shader, source ):
	"""Python-specific version of the shader call
	
	shader -- shader object as created by glCreateShaderObjectARB
	source -- Python source-code string or sequence of strings
	
	Note: we hack this somewhat to make it easier to deal with
	from Python.  We allow for lists-of-strings, but we just 
	join them into a single string before passing them in...
	"""
	if not isinstance( source, str ):
		source = '\n'.join( source )
	# simple case...
	data = ctypes.c_char_p( source )
	arraySize = len(source)
	length = ctypes.c_int(arraySize)
	return base_glShaderSourceARB(
		shader, 1, 
		ctypes.cast( ctypes.POINTER( constants.GLcharARB ), data,  ),
		ctypes.byref( length) 
	)




'''OpenGL extension SGIS.generate_mipmap

Overview (from the spec)
	
	This extension defines a mechanism by which OpenGL can derive the
	entire set of mipmap arrays when provided with only the base level
	array.  Automatic mipmap generation is particularly useful when
	texture images are being provided as a video stream.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIS/generate_mipmap.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIS_generate_mipmap'
GL_GENERATE_MIPMAP_SGIS = constant.Constant( 'GL_GENERATE_MIPMAP_SGIS', 0x8191 )
GL_GENERATE_MIPMAP_HINT_SGIS = constant.Constant( 'GL_GENERATE_MIPMAP_HINT_SGIS', 0x8192 )
glget.addGLGetConstant( GL_GENERATE_MIPMAP_HINT_SGIS, (1,) )


def glInitGenerateMipmapSGIS():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.transform_feedback

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/transform_feedback.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_transform_feedback'
GL_TRANSFORM_FEEDBACK_BUFFER_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_EXT', 0x8C8E )
GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT', 0x8C84 )
GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT', 0x8C85 )
GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT', 0x8C8F )
GL_INTERLEAVED_ATTRIBS_EXT = constant.Constant( 'GL_INTERLEAVED_ATTRIBS_EXT', 0x8C8C )
GL_SEPARATE_ATTRIBS_EXT = constant.Constant( 'GL_SEPARATE_ATTRIBS_EXT', 0x8C8D )
GL_PRIMITIVES_GENERATED_EXT = constant.Constant( 'GL_PRIMITIVES_GENERATED_EXT', 0x8C87 )
GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT', 0x8C88 )
GL_RASTERIZER_DISCARD_EXT = constant.Constant( 'GL_RASTERIZER_DISCARD_EXT', 0x8C89 )
GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT', 0x8C8A )
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT', 0x8C8B )
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT', 0x8C80 )
GL_TRANSFORM_FEEDBACK_VARYINGS_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_VARYINGS_EXT', 0x8C83 )
GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT', 0x8C7F )
GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT', 0x8C76 )
glBeginTransformFeedbackEXT = platform.createExtensionFunction( 
	'glBeginTransformFeedbackEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glBeginTransformFeedbackEXT( GLenum(primitiveMode) ) -> None',
	argNames = ('primitiveMode',),
)

glEndTransformFeedbackEXT = platform.createExtensionFunction( 
	'glEndTransformFeedbackEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(),
	doc = 'glEndTransformFeedbackEXT(  ) -> None',
	argNames = (),
)

glBindBufferRangeEXT = platform.createExtensionFunction( 
	'glBindBufferRangeEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLintptr, constants.GLsizeiptr,),
	doc = 'glBindBufferRangeEXT( GLenum(target), GLuint(index), GLuint(buffer), GLintptr(offset), GLsizeiptr(size) ) -> None',
	argNames = ('target', 'index', 'buffer', 'offset', 'size',),
)

glBindBufferOffsetEXT = platform.createExtensionFunction( 
	'glBindBufferOffsetEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLintptr,),
	doc = 'glBindBufferOffsetEXT( GLenum(target), GLuint(index), GLuint(buffer), GLintptr(offset) ) -> None',
	argNames = ('target', 'index', 'buffer', 'offset',),
)

glBindBufferBaseEXT = platform.createExtensionFunction( 
	'glBindBufferBaseEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint,),
	doc = 'glBindBufferBaseEXT( GLenum(target), GLuint(index), GLuint(buffer) ) -> None',
	argNames = ('target', 'index', 'buffer',),
)

glTransformFeedbackVaryingsEXT = platform.createExtensionFunction( 
	'glTransformFeedbackVaryingsEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLsizei, arrays.GLintArray, constants.GLenum,),
	doc = 'glTransformFeedbackVaryingsEXT( GLuint(program), GLsizei(count), GLintArray(locations), GLenum(bufferMode) ) -> None',
	argNames = ('program', 'count', 'locations', 'bufferMode',),
)

glGetTransformFeedbackVaryingEXT = platform.createExtensionFunction( 
	'glGetTransformFeedbackVaryingEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, arrays.GLintArray,),
	doc = 'glGetTransformFeedbackVaryingEXT( GLuint(program), GLuint(index), GLintArray(location) ) -> None',
	argNames = ('program', 'index', 'location',),
)


def glInitTransformFeedbackEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension HP.texture_lighting

Overview (from the spec)
	
	This extension defines a mechanism for applications to request
	that color originating from specular lighting be added to
	the fragment color _after_ texture application.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/HP/texture_lighting.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_HP_texture_lighting'
GL_TEXTURE_LIGHTING_MODE_HP = constant.Constant( 'GL_TEXTURE_LIGHTING_MODE_HP', 0x8167 )
GL_TEXTURE_POST_SPECULAR_HP = constant.Constant( 'GL_TEXTURE_POST_SPECULAR_HP', 0x8168 )
GL_TEXTURE_PRE_SPECULAR_HP = constant.Constant( 'GL_TEXTURE_PRE_SPECULAR_HP', 0x8169 )


def glInitTextureLightingHP():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.fog_scale

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIX/fog_scale.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_fog_scale'
GL_FOG_SCALE_SGIX = constant.Constant( 'GL_FOG_SCALE_SGIX', 0x81FC )
GL_FOG_SCALE_VALUE_SGIX = constant.Constant( 'GL_FOG_SCALE_VALUE_SGIX', 0x81FD )


def glInitFogScaleSGIX():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.draw_range_elements

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/draw_range_elements.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_draw_range_elements'
GL_MAX_ELEMENTS_VERTICES_EXT = constant.Constant( 'GL_MAX_ELEMENTS_VERTICES_EXT', 0x80E8 )
GL_MAX_ELEMENTS_INDICES_EXT = constant.Constant( 'GL_MAX_ELEMENTS_INDICES_EXT', 0x80E9 )
glDrawRangeElementsEXT = platform.createExtensionFunction( 
	'glDrawRangeElementsEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLsizei, constants.GLenum, ctypes.c_void_p,),
	doc = 'glDrawRangeElementsEXT( GLenum(mode), GLuint(start), GLuint(end), GLsizei(count), GLenum(type), c_void_p(indices) ) -> None',
	argNames = ('mode', 'start', 'end', 'count', 'type', 'indices',),
)


def glInitDrawRangeElementsEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Exceptional cases that need some extra wrapping"""
from OpenGL.platform import GL,GLU,createBaseFunction
from OpenGL import arrays, error, wrapper
from OpenGL.arrays.arraydatatype import GLfloatArray
from OpenGL import constants as data_types
from OpenGL.lazywrapper import lazy
from OpenGL.raw import GL as simple
from OpenGL.raw.GL import constants
from OpenGL.raw.GL import annotations
import OpenGL
import ctypes

__all__ = [
	'glBegin', 
	'glCallLists', 
	'glColor', 
	#'glColorTableParameterfv', 
	'glDeleteTextures', 
	'glEdgeFlagv', 
	'glEnd', 
	'glGenTextures', 
	'glIndexdv', 
	'glIndexfv', 
	'glIndexsv',
	'glIndexubv', 
	'glMap1d', 
	'glMap1f',
	'glMap2d',
	'glMap2f',
	'glMaterial', 
	'glRasterPos', 
	'glRectfv', 
	'glRectiv', 
	'glRectsv', 
	'glTexGenfv', 
	'glTexParameter', 
	'glVertex', 
	'glAreTexturesResident',
]

glRasterPosDispatch = {
	2: simple.glRasterPos2dv,
	3: simple.glRasterPos3dv,
	4: simple.glRasterPos4dv,
}

if OpenGL.ERROR_CHECKING:
	@lazy( simple.glBegin )
	def glBegin( baseFunction, mode ):
		"""Begin GL geometry-definition mode, disable automatic error checking"""
		error.onBegin( )
		return baseFunction( mode )
	@lazy( simple.glEnd )
	def glEnd( baseFunction ):
		"""Finish GL geometry-definition mode, re-enable automatic error checking"""
		error.onEnd( )
		return baseFunction( )
else:
	glBegin = simple.glBegin
	glEnd = simple.glEnd

@lazy( simple.glDeleteTextures )
def glDeleteTextures( baseFunction, array ):
	"""Delete specified set of textures"""
	ptr = arrays.GLuintArray.asArray( array )
	size = arrays.GLuintArray.arraySize( ptr )
	return baseFunction( size, ptr )


def glMap2( baseFunction, arrayType ):
	def glMap2( target, u1, u2, v1, v2, points):
		"""glMap2(target, u1, u2, v1, v2, points[][][]) -> None
		
		This is a completely non-standard signature which doesn't allow for most 
		of the funky uses with strides and the like, but it has been like this for
		a very long time...
		"""
		ptr = arrayType.asArray( points )
		uorder,vorder,vstride = arrayType.dimensions( ptr )
		ustride = vstride*vorder
		return baseFunction(
			target, 
			u1, u2, 
			ustride, uorder, 
			v1, v2, 
			vstride, vorder, 
			ptr
		)
	glMap2.__name__ = baseFunction.__name__
	glMap2.baseFunction = baseFunction
	return glMap2
glMap2d = glMap2( simple.glMap2d, arrays.GLdoubleArray )
glMap2f = glMap2( simple.glMap2f, arrays.GLfloatArray )
del glMap2

def glMap1( baseFunction, arrayType ):
	def glMap1(target,u1,u2,points):
		"""glMap1(target, u1, u2, points[][][]) -> None
		
		This is a completely non-standard signature which doesn't allow for most 
		of the funky uses with strides and the like, but it has been like this for
		a very long time...
		"""
		ptr = arrayType.asArray( points )
		dims = arrayType.dimensions( ptr )
		uorder = dims[0]
		ustride = dims[1]
		return baseFunction( target, u1,u2,ustride,uorder, ptr )
	glMap1.__name__ == baseFunction.__name__
	glMap1.baseFunction = baseFunction
	return glMap1
glMap1d = glMap1( simple.glMap1d, arrays.GLdoubleArray )
glMap1f = glMap1( simple.glMap1f, arrays.GLfloatArray )
del glMap1

def glRasterPos( *args ):
	"""Choose glRasterPosX based on number of args"""
	if len(args) == 1:
		# v form...
		args = args[0]
	return glRasterPosDispatch[ len(args) ]( args )

glVertexDispatch = {
	2: simple.glVertex2dv,
	3: simple.glVertex3dv,
	4: simple.glVertex4dv,
}
def glVertex( *args ):
	"""Choose glVertexX based on number of args"""
	if len(args) == 1:
		# v form...
		args = args[0]
	return glVertexDispatch[ len(args) ]( args )

@lazy( simple.glCallLists )
def glCallLists( baseFunction, lists ):
	"""glCallLists( str( lists ) or lists[] ) -> None 
	
	Restricted version of glCallLists, takes a string or a GLuint compatible
	array data-type and passes into the base function.
	"""
	if isinstance( lists, str ):
		return baseFunction(
			len(lists),
			constants.GL_UNSIGNED_BYTE,
			ctypes.c_void_p(arrays.GLubyteArray.dataPointer( lists )),
		)
	ptr = arrays.GLuintArray.asArray( lists )
	size = arrays.GLuintArray.arraySize( ptr )
	return baseFunction( 
		size, 
		constants.GL_UNSIGNED_INT, 
		ctypes.c_void_p( arrays.GLuintArray.dataPointer(ptr))
	)

def glTexParameter( target, pname, parameter ):
	"""Set a texture parameter, choose underlying call based on pname and parameter"""
	if isinstance( parameter, float ):
		return simple.glTexParameterf( target, pname, parameter )
	elif isinstance( parameter, int ):
		return simple.glTexParameteri( target, pname, parameter )
	else:
		return simple.glTexParameterfv( target, pname, parameter )

@lazy( simple.glGenTextures )
def glGenTextures( baseFunction, count, textures=None ):
	"""Generate count new texture names
	
	Note: for compatibility with PyOpenGL 2.x and below,
	a count of 1 will return a single integer, rather than 
	an array of integers.
	"""
	if count <= 0:
		raise ValueError( """Can't generate 0 or fewer textures""" )
	elif count == 1:
		# this traditionally returned a single int/long, so we'll continue to
		# do so, even though it would be easier not to bother.
		textures = simple.GLuint( 0 )
		baseFunction( count, textures)
		return textures.value
	else:
		textures = arrays.GLuintArray.zeros( (count,))
		baseFunction( count, textures)
		return textures

def glMaterial( faces, constant, *args ):
	"""glMaterial -- convenience function to dispatch on argument type
	
	If passed a single argument in args, calls:
		glMaterialfv( faces, constant, args[0] )
	else calls:
		glMaterialf( faces, constant, *args )
	"""
	if len(args) == 1:
		arg = GLfloatArray.asArray( args[0] )
		if arg is None:
			raise ValueError( """Null value in glMaterial: %s"""%(args,) )
		return simple.glMaterialfv( faces, constant, arg )
	else:
		return simple.glMaterialf( faces, constant, *args )

glColorDispatch = {
	3: annotations.glColor3dv,
	4: annotations.glColor4dv,
}

def glColor( *args ):
	"""glColor*f* -- convenience function to dispatch on argument type

	dispatches to glColor3f, glColor2f, glColor4f, glColor3f, glColor2f, glColor4f
	depending on the arguments passed...
	"""
	arglen = len(args)
	if arglen == 1:
		arg = arrays.GLfloatArray.asArray( args[0] )
		function = glColorDispatch[arrays.GLfloatArray.arraySize( arg )]
		return function( arg )
	elif arglen == 2:
		return simple.glColor2d( *args )
	elif arglen == 3:
		return simple.glColor3d( *args )
	elif arglen == 4:
		return simple.glColor4d( *args )
	else:
		raise ValueError( """Don't know how to handle arguments: %s"""%(args,))


# Rectagle coordinates,
glRectfv = arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
		simple.glRectfv,
		2,
		arrays.GLfloatArray,
		'v1',
	),
	2,
	arrays.GLfloatArray,
	'v2',
)
glRectiv = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		simple.glRectiv,
		2,
		arrays.GLintArray,
		'v1',
	),
	2,
	arrays.GLintArray,
	'v2',
)
glRectsv = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		simple.glRectsv,
		2,
		arrays.GLshortArray,
		'v1',
	),
	2,
	arrays.GLshortArray,
	'v2',
)


glIndexsv = arrays.setInputArraySizeType(
	simple.glIndexsv,
	1,
	arrays.GLshortArray,
	'c',
)
glIndexdv = arrays.setInputArraySizeType(
	simple.glIndexdv,
	1,
	arrays.GLdoubleArray,
	'c',
)
glIndexfv = arrays.setInputArraySizeType(
	simple.glIndexfv,
	1,
	arrays.GLfloatArray,
	'c',
)
glIndexubv = arrays.setInputArraySizeType(
	simple.glIndexubv,
	1,
	arrays.GLbyteArray,
	'c',
)
glEdgeFlagv = arrays.setInputArraySizeType(
	simple.glEdgeFlagv,
	1,
	arrays.GLubyteArray,
	'flag',
)
glTexGenfv = arrays.setInputArraySizeType(
	simple.glTexGenfv,
	None,
	arrays.GLfloatArray,
	'params',
)

#'glAreTexturesResident',
@lazy( simple.glAreTexturesResident )
def glAreTexturesResident( baseFunction, *args ):
	"""Allow both Pythonic and C-style calls to glAreTexturesResident
	
		glAreTexturesResident( arrays.GLuintArray( textures) )
	
	or 
	
		glAreTexturesResident( int(n), arrays.GLuintArray( textures), arrays.GLuboolean( output) )
	
	or 
	
		glAreTexturesResident( int(n), arrays.GLuintArray( textures) )
		
	returns the output arrays.GLubooleanArray
	"""
	if len(args) == 1:
		# Pythonic form...
		textures = args[0]
		textures = arrays.GLuintArray.asArray( textures )
		n = arrays.GLuintArray.arraySize(textures)
		output = arrays.GLbooleanArray.zeros( (n,))
	elif len(args) == 2:
		try:
			n = int( args[0] )
		except TypeError, err:
			textures = args[0]
			textures = arrays.GLuintArray.asArray( textures )

			n = arrays.GLuintArray.arraySize(textures)
			output = args[1]
			output = arrays.GLbooleanArray.asArray( output )
		else:
			textures = args[1]
			textures = arrays.GLuintArray.asArray( textures )

			output = arrays.GLbooleanArray.zeros( (n,))
	elif len(args) == 3:
		n,textures,output = args 
		textures = arrays.GLuintArray.asArray( textures )
		output = arrays.GLbooleanArray.asArray( output )
	else:
		raise TypeError( """Expected 1 to 3 arguments to glAreTexturesResident""" )
	texturePtr = arrays.GLuintArray.typedPointer( textures )
	outputPtr = arrays.GLbooleanArray.typedPointer( output )
	result = baseFunction( n, texturePtr, outputPtr )
	if result:
		# weirdness of the C api, doesn't produce values if all are true
		for i in range(len(output)):
			output[i] = 1
	return output

#glMap2f
#glMap2d
#glMap1f
#glMap1d
#glPixelMapusv
#glTexGenfv
#glLightfv
#glFeedbackBuffer
#glDrawRangeElements
#glSelectBuffer
#glAreTexturesResident
#glPixelMapfv
#glTexGeniv
#glClipPlane
#glTexParameterfv
#glTexParameteriv
#glReadPixels
#glConvolutionParameterfv
#glPolygonStipple
#glFogiv
#glTexEnviv
#glRectdv
#glMaterialiv
#glColorTable
#glColorTableParameteriv
#glIndexiv
#glLightModeliv
#glDrawElements
#glConvolutionFilter1D
#glCallLists



"""Array-size annotations for OpenGL.raw.GL

Automatically generated by the generateraw script, do not edit!
"""
from OpenGL.raw import GL as raw

from ctypes import *
from OpenGL import platform, arrays
from OpenGL.constant import Constant
from OpenGL import constants as GLconstants
GLvoid = GLconstants.GLvoid



glAreTexturesResident = arrays.setInputArraySizeType(
	raw.glAreTexturesResident,
	None, # XXX Could not determine size of argument textures for glAreTexturesResident arrays.GLuintArray
	arrays.GLuintArray, 
	'textures',
)

glBitmap = arrays.setInputArraySizeType(
	raw.glBitmap,
	None, # XXX Could not determine size of argument bitmap for glBitmap arrays.GLubyteArray
	arrays.GLubyteArray, 
	'bitmap',
)

glClipPlane = arrays.setInputArraySizeType(
	raw.glClipPlane,
	None, # XXX Could not determine size of argument equation for glClipPlane arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'equation',
)

glColor3bv = arrays.setInputArraySizeType(
	raw.glColor3bv,
	3,
	arrays.GLbyteArray,
	'v',
)

glColor3dv = arrays.setInputArraySizeType(
	raw.glColor3dv,
	3,
	arrays.GLdoubleArray,
	'v',
)

glColor3fv = arrays.setInputArraySizeType(
	raw.glColor3fv,
	3,
	arrays.GLfloatArray,
	'v',
)

glColor3iv = arrays.setInputArraySizeType(
	raw.glColor3iv,
	3,
	arrays.GLintArray,
	'v',
)

glColor3sv = arrays.setInputArraySizeType(
	raw.glColor3sv,
	3,
	arrays.GLshortArray,
	'v',
)

glColor3ubv = arrays.setInputArraySizeType(
	raw.glColor3ubv,
	3,
	arrays.GLubyteArray,
	'v',
)

glColor3uiv = arrays.setInputArraySizeType(
	raw.glColor3uiv,
	3,
	arrays.GLuintArray,
	'v',
)

glColor3usv = arrays.setInputArraySizeType(
	raw.glColor3usv,
	3,
	arrays.GLushortArray,
	'v',
)

glColor4bv = arrays.setInputArraySizeType(
	raw.glColor4bv,
	4,
	arrays.GLbyteArray,
	'v',
)

glColor4dv = arrays.setInputArraySizeType(
	raw.glColor4dv,
	4,
	arrays.GLdoubleArray,
	'v',
)

glColor4fv = arrays.setInputArraySizeType(
	raw.glColor4fv,
	4,
	arrays.GLfloatArray,
	'v',
)

glColor4iv = arrays.setInputArraySizeType(
	raw.glColor4iv,
	4,
	arrays.GLintArray,
	'v',
)

glColor4sv = arrays.setInputArraySizeType(
	raw.glColor4sv,
	4,
	arrays.GLshortArray,
	'v',
)

glColor4ubv = arrays.setInputArraySizeType(
	raw.glColor4ubv,
	4,
	arrays.GLubyteArray,
	'v',
)

glColor4uiv = arrays.setInputArraySizeType(
	raw.glColor4uiv,
	4,
	arrays.GLuintArray,
	'v',
)

glColor4usv = arrays.setInputArraySizeType(
	raw.glColor4usv,
	4,
	arrays.GLushortArray,
	'v',
)

glDeleteTextures = arrays.setInputArraySizeType(
	raw.glDeleteTextures,
	None, # XXX Could not determine size of argument textures for glDeleteTextures arrays.GLuintArray
	arrays.GLuintArray, 
	'textures',
)

glEvalCoord1dv = arrays.setInputArraySizeType(
	raw.glEvalCoord1dv,
	1,
	arrays.GLdoubleArray,
	'u',
)

glEvalCoord1fv = arrays.setInputArraySizeType(
	raw.glEvalCoord1fv,
	1,
	arrays.GLfloatArray,
	'u',
)

glEvalCoord2dv = arrays.setInputArraySizeType(
	raw.glEvalCoord2dv,
	2,
	arrays.GLdoubleArray,
	'u',
)

glEvalCoord2fv = arrays.setInputArraySizeType(
	raw.glEvalCoord2fv,
	2,
	arrays.GLfloatArray,
	'u',
)

glFeedbackBuffer = arrays.setInputArraySizeType(
	raw.glFeedbackBuffer,
	None, # XXX Could not determine size of argument buffer for glFeedbackBuffer arrays.GLfloatArray
	arrays.GLfloatArray, 
	'buffer',
)

glFogfv = arrays.setInputArraySizeType(
	raw.glFogfv,
	None, # XXX Could not determine size of argument params for glFogfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glFogiv = arrays.setInputArraySizeType(
	raw.glFogiv,
	None, # XXX Could not determine size of argument params for glFogiv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glGenTextures = arrays.setInputArraySizeType(
	raw.glGenTextures,
	None, # XXX Could not determine size of argument textures for glGenTextures arrays.GLuintArray
	arrays.GLuintArray, 
	'textures',
)

glGetClipPlane = arrays.setInputArraySizeType(
	raw.glGetClipPlane,
	None, # XXX Could not determine size of argument equation for glGetClipPlane arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'equation',
)

glGetDoublev = arrays.setInputArraySizeType(
	raw.glGetDoublev,
	None, # XXX Could not determine size of argument params for glGetDoublev arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'params',
)

glGetFloatv = arrays.setInputArraySizeType(
	raw.glGetFloatv,
	None, # XXX Could not determine size of argument params for glGetFloatv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glGetIntegerv = arrays.setInputArraySizeType(
	raw.glGetIntegerv,
	None, # XXX Could not determine size of argument params for glGetIntegerv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glGetLightfv = arrays.setInputArraySizeType(
	raw.glGetLightfv,
	None, # XXX Could not determine size of argument params for glGetLightfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glGetLightiv = arrays.setInputArraySizeType(
	raw.glGetLightiv,
	None, # XXX Could not determine size of argument params for glGetLightiv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glGetMapdv = arrays.setInputArraySizeType(
	raw.glGetMapdv,
	None, # XXX Could not determine size of argument v for glGetMapdv arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'v',
)

glGetMapfv = arrays.setInputArraySizeType(
	raw.glGetMapfv,
	None, # XXX Could not determine size of argument v for glGetMapfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'v',
)

glGetMapiv = arrays.setInputArraySizeType(
	raw.glGetMapiv,
	None, # XXX Could not determine size of argument v for glGetMapiv arrays.GLintArray
	arrays.GLintArray, 
	'v',
)

glGetMaterialfv = arrays.setInputArraySizeType(
	raw.glGetMaterialfv,
	None, # XXX Could not determine size of argument params for glGetMaterialfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glGetMaterialiv = arrays.setInputArraySizeType(
	raw.glGetMaterialiv,
	None, # XXX Could not determine size of argument params for glGetMaterialiv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glGetPixelMapfv = arrays.setInputArraySizeType(
	raw.glGetPixelMapfv,
	None, # XXX Could not determine size of argument values for glGetPixelMapfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'values',
)

glGetPixelMapuiv = arrays.setInputArraySizeType(
	raw.glGetPixelMapuiv,
	None, # XXX Could not determine size of argument values for glGetPixelMapuiv arrays.GLuintArray
	arrays.GLuintArray, 
	'values',
)

glGetPixelMapusv = arrays.setInputArraySizeType(
	raw.glGetPixelMapusv,
	None, # XXX Could not determine size of argument values for glGetPixelMapusv arrays.GLushortArray
	arrays.GLushortArray, 
	'values',
)

glGetPolygonStipple = arrays.setInputArraySizeType(
	raw.glGetPolygonStipple,
	None, # XXX Could not determine size of argument mask for glGetPolygonStipple arrays.GLubyteArray
	arrays.GLubyteArray, 
	'mask',
)

glGetTexEnvfv = arrays.setInputArraySizeType(
	raw.glGetTexEnvfv,
	None, # XXX Could not determine size of argument params for glGetTexEnvfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glGetTexEnviv = arrays.setInputArraySizeType(
	raw.glGetTexEnviv,
	None, # XXX Could not determine size of argument params for glGetTexEnviv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glGetTexGendv = arrays.setInputArraySizeType(
	raw.glGetTexGendv,
	None, # XXX Could not determine size of argument params for glGetTexGendv arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'params',
)

glGetTexGenfv = arrays.setInputArraySizeType(
	raw.glGetTexGenfv,
	None, # XXX Could not determine size of argument params for glGetTexGenfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glGetTexGeniv = arrays.setInputArraySizeType(
	raw.glGetTexGeniv,
	None, # XXX Could not determine size of argument params for glGetTexGeniv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glGetTexLevelParameterfv = arrays.setInputArraySizeType(
	raw.glGetTexLevelParameterfv,
	None, # XXX Could not determine size of argument params for glGetTexLevelParameterfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glGetTexLevelParameteriv = arrays.setInputArraySizeType(
	raw.glGetTexLevelParameteriv,
	None, # XXX Could not determine size of argument params for glGetTexLevelParameteriv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glGetTexParameterfv = arrays.setInputArraySizeType(
	raw.glGetTexParameterfv,
	None, # XXX Could not determine size of argument params for glGetTexParameterfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glGetTexParameteriv = arrays.setInputArraySizeType(
	raw.glGetTexParameteriv,
	None, # XXX Could not determine size of argument params for glGetTexParameteriv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glIndexdv = arrays.setInputArraySizeType(
	raw.glIndexdv,
	None, # XXX Could not determine size of argument c for glIndexdv arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'c',
)

glIndexfv = arrays.setInputArraySizeType(
	raw.glIndexfv,
	None, # XXX Could not determine size of argument c for glIndexfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'c',
)

glIndexiv = arrays.setInputArraySizeType(
	raw.glIndexiv,
	None, # XXX Could not determine size of argument c for glIndexiv arrays.GLintArray
	arrays.GLintArray, 
	'c',
)

glIndexsv = arrays.setInputArraySizeType(
	raw.glIndexsv,
	None, # XXX Could not determine size of argument c for glIndexsv arrays.GLshortArray
	arrays.GLshortArray, 
	'c',
)

glIndexubv = arrays.setInputArraySizeType(
	raw.glIndexubv,
	None, # XXX Could not determine size of argument c for glIndexubv arrays.GLubyteArray
	arrays.GLubyteArray, 
	'c',
)

glLightModelfv = arrays.setInputArraySizeType(
	raw.glLightModelfv,
	None, # XXX Could not determine size of argument params for glLightModelfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glLightModeliv = arrays.setInputArraySizeType(
	raw.glLightModeliv,
	None, # XXX Could not determine size of argument params for glLightModeliv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glLightfv = arrays.setInputArraySizeType(
	raw.glLightfv,
	None, # XXX Could not determine size of argument params for glLightfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glLightiv = arrays.setInputArraySizeType(
	raw.glLightiv,
	None, # XXX Could not determine size of argument params for glLightiv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glLoadMatrixd = arrays.setInputArraySizeType(
	raw.glLoadMatrixd,
	16,
	arrays.GLdoubleArray,
	'm',
)

glLoadMatrixf = arrays.setInputArraySizeType(
	raw.glLoadMatrixf,
	16,
	arrays.GLfloatArray,
	'm',
)

glMap1d = arrays.setInputArraySizeType(
	raw.glMap1d,
	None, # XXX Could not determine size of argument points for glMap1d arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'points',
)

glMap1f = arrays.setInputArraySizeType(
	raw.glMap1f,
	None, # XXX Could not determine size of argument points for glMap1f arrays.GLfloatArray
	arrays.GLfloatArray, 
	'points',
)

glMap2d = arrays.setInputArraySizeType(
	raw.glMap2d,
	None, # XXX Could not determine size of argument points for glMap2d arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'points',
)

glMap2f = arrays.setInputArraySizeType(
	raw.glMap2f,
	None, # XXX Could not determine size of argument points for glMap2f arrays.GLfloatArray
	arrays.GLfloatArray, 
	'points',
)

glMaterialfv = arrays.setInputArraySizeType(
	raw.glMaterialfv,
	None, # XXX Could not determine size of argument params for glMaterialfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glMaterialiv = arrays.setInputArraySizeType(
	raw.glMaterialiv,
	None, # XXX Could not determine size of argument params for glMaterialiv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glMultMatrixd = arrays.setInputArraySizeType(
	raw.glMultMatrixd,
	16,
	arrays.GLdoubleArray,
	'm',
)

glMultMatrixf = arrays.setInputArraySizeType(
	raw.glMultMatrixf,
	16,
	arrays.GLfloatArray,
	'm',
)

glNormal3bv = arrays.setInputArraySizeType(
	raw.glNormal3bv,
	3,
	arrays.GLbyteArray,
	'v',
)

glNormal3dv = arrays.setInputArraySizeType(
	raw.glNormal3dv,
	3,
	arrays.GLdoubleArray,
	'v',
)

glNormal3fv = arrays.setInputArraySizeType(
	raw.glNormal3fv,
	3,
	arrays.GLfloatArray,
	'v',
)

glNormal3iv = arrays.setInputArraySizeType(
	raw.glNormal3iv,
	3,
	arrays.GLintArray,
	'v',
)

glNormal3sv = arrays.setInputArraySizeType(
	raw.glNormal3sv,
	3,
	arrays.GLshortArray,
	'v',
)

glPixelMapfv = arrays.setInputArraySizeType(
	raw.glPixelMapfv,
	None, # XXX Could not determine size of argument values for glPixelMapfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'values',
)

glPixelMapuiv = arrays.setInputArraySizeType(
	raw.glPixelMapuiv,
	None, # XXX Could not determine size of argument values for glPixelMapuiv arrays.GLuintArray
	arrays.GLuintArray, 
	'values',
)

glPixelMapusv = arrays.setInputArraySizeType(
	raw.glPixelMapusv,
	None, # XXX Could not determine size of argument values for glPixelMapusv arrays.GLushortArray
	arrays.GLushortArray, 
	'values',
)

glPolygonStipple = arrays.setInputArraySizeType(
	raw.glPolygonStipple,
	None, # XXX Could not determine size of argument mask for glPolygonStipple arrays.GLubyteArray
	arrays.GLubyteArray, 
	'mask',
)

glPrioritizeTextures = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		raw.glPrioritizeTextures,
		None, # XXX Could not determine size of argument textures for glPrioritizeTextures arrays.GLuintArray
		arrays.GLuintArray, 
		'textures',
	),
	None, # XXX Could not determine size of argument priorities for glPrioritizeTextures arrays.GLclampfArray
	arrays.GLclampfArray, 
	'priorities',
)

glRasterPos2dv = arrays.setInputArraySizeType(
	raw.glRasterPos2dv,
	2,
	arrays.GLdoubleArray,
	'v',
)

glRasterPos2fv = arrays.setInputArraySizeType(
	raw.glRasterPos2fv,
	2,
	arrays.GLfloatArray,
	'v',
)

glRasterPos2iv = arrays.setInputArraySizeType(
	raw.glRasterPos2iv,
	2,
	arrays.GLintArray,
	'v',
)

glRasterPos2sv = arrays.setInputArraySizeType(
	raw.glRasterPos2sv,
	2,
	arrays.GLshortArray,
	'v',
)

glRasterPos3dv = arrays.setInputArraySizeType(
	raw.glRasterPos3dv,
	3,
	arrays.GLdoubleArray,
	'v',
)

glRasterPos3fv = arrays.setInputArraySizeType(
	raw.glRasterPos3fv,
	3,
	arrays.GLfloatArray,
	'v',
)

glRasterPos3iv = arrays.setInputArraySizeType(
	raw.glRasterPos3iv,
	3,
	arrays.GLintArray,
	'v',
)

glRasterPos3sv = arrays.setInputArraySizeType(
	raw.glRasterPos3sv,
	3,
	arrays.GLshortArray,
	'v',
)

glRasterPos4dv = arrays.setInputArraySizeType(
	raw.glRasterPos4dv,
	4,
	arrays.GLdoubleArray,
	'v',
)

glRasterPos4fv = arrays.setInputArraySizeType(
	raw.glRasterPos4fv,
	4,
	arrays.GLfloatArray,
	'v',
)

glRasterPos4iv = arrays.setInputArraySizeType(
	raw.glRasterPos4iv,
	4,
	arrays.GLintArray,
	'v',
)

glRasterPos4sv = arrays.setInputArraySizeType(
	raw.glRasterPos4sv,
	4,
	arrays.GLshortArray,
	'v',
)

glRectdv = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		raw.glRectdv,
		None, # XXX Could not determine size of argument v1 for glRectdv arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'v1',
	),
	None, # XXX Could not determine size of argument v2 for glRectdv arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'v2',
)

glRectfv = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		raw.glRectfv,
		None, # XXX Could not determine size of argument v1 for glRectfv arrays.GLfloatArray
		arrays.GLfloatArray, 
		'v1',
	),
	None, # XXX Could not determine size of argument v2 for glRectfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'v2',
)

glRectiv = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		raw.glRectiv,
		None, # XXX Could not determine size of argument v1 for glRectiv arrays.GLintArray
		arrays.GLintArray, 
		'v1',
	),
	None, # XXX Could not determine size of argument v2 for glRectiv arrays.GLintArray
	arrays.GLintArray, 
	'v2',
)

glRectsv = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		raw.glRectsv,
		None, # XXX Could not determine size of argument v1 for glRectsv arrays.GLshortArray
		arrays.GLshortArray, 
		'v1',
	),
	None, # XXX Could not determine size of argument v2 for glRectsv arrays.GLshortArray
	arrays.GLshortArray, 
	'v2',
)

glSelectBuffer = arrays.setInputArraySizeType(
	raw.glSelectBuffer,
	None, # XXX Could not determine size of argument buffer for glSelectBuffer arrays.GLuintArray
	arrays.GLuintArray, 
	'buffer',
)

glTexCoord1dv = arrays.setInputArraySizeType(
	raw.glTexCoord1dv,
	1,
	arrays.GLdoubleArray,
	'v',
)

glTexCoord1fv = arrays.setInputArraySizeType(
	raw.glTexCoord1fv,
	1,
	arrays.GLfloatArray,
	'v',
)

glTexCoord1iv = arrays.setInputArraySizeType(
	raw.glTexCoord1iv,
	1,
	arrays.GLintArray,
	'v',
)

glTexCoord1sv = arrays.setInputArraySizeType(
	raw.glTexCoord1sv,
	1,
	arrays.GLshortArray,
	'v',
)

glTexCoord2dv = arrays.setInputArraySizeType(
	raw.glTexCoord2dv,
	2,
	arrays.GLdoubleArray,
	'v',
)

glTexCoord2fv = arrays.setInputArraySizeType(
	raw.glTexCoord2fv,
	2,
	arrays.GLfloatArray,
	'v',
)

glTexCoord2iv = arrays.setInputArraySizeType(
	raw.glTexCoord2iv,
	2,
	arrays.GLintArray,
	'v',
)

glTexCoord2sv = arrays.setInputArraySizeType(
	raw.glTexCoord2sv,
	2,
	arrays.GLshortArray,
	'v',
)

glTexCoord3dv = arrays.setInputArraySizeType(
	raw.glTexCoord3dv,
	3,
	arrays.GLdoubleArray,
	'v',
)

glTexCoord3fv = arrays.setInputArraySizeType(
	raw.glTexCoord3fv,
	3,
	arrays.GLfloatArray,
	'v',
)

glTexCoord3iv = arrays.setInputArraySizeType(
	raw.glTexCoord3iv,
	3,
	arrays.GLintArray,
	'v',
)

glTexCoord3sv = arrays.setInputArraySizeType(
	raw.glTexCoord3sv,
	3,
	arrays.GLshortArray,
	'v',
)

glTexCoord4dv = arrays.setInputArraySizeType(
	raw.glTexCoord4dv,
	4,
	arrays.GLdoubleArray,
	'v',
)

glTexCoord4fv = arrays.setInputArraySizeType(
	raw.glTexCoord4fv,
	4,
	arrays.GLfloatArray,
	'v',
)

glTexCoord4iv = arrays.setInputArraySizeType(
	raw.glTexCoord4iv,
	4,
	arrays.GLintArray,
	'v',
)

glTexCoord4sv = arrays.setInputArraySizeType(
	raw.glTexCoord4sv,
	4,
	arrays.GLshortArray,
	'v',
)

glTexEnvfv = arrays.setInputArraySizeType(
	raw.glTexEnvfv,
	None, # XXX Could not determine size of argument params for glTexEnvfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glTexEnviv = arrays.setInputArraySizeType(
	raw.glTexEnviv,
	None, # XXX Could not determine size of argument params for glTexEnviv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glTexGendv = arrays.setInputArraySizeType(
	raw.glTexGendv,
	None, # XXX Could not determine size of argument params for glTexGendv arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'params',
)

glTexGenfv = arrays.setInputArraySizeType(
	raw.glTexGenfv,
	None, # XXX Could not determine size of argument params for glTexGenfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glTexGeniv = arrays.setInputArraySizeType(
	raw.glTexGeniv,
	None, # XXX Could not determine size of argument params for glTexGeniv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glTexParameterfv = arrays.setInputArraySizeType(
	raw.glTexParameterfv,
	None, # XXX Could not determine size of argument params for glTexParameterfv arrays.GLfloatArray
	arrays.GLfloatArray, 
	'params',
)

glTexParameteriv = arrays.setInputArraySizeType(
	raw.glTexParameteriv,
	None, # XXX Could not determine size of argument params for glTexParameteriv arrays.GLintArray
	arrays.GLintArray, 
	'params',
)

glVertex2dv = arrays.setInputArraySizeType(
	raw.glVertex2dv,
	2,
	arrays.GLdoubleArray,
	'v',
)

glVertex2fv = arrays.setInputArraySizeType(
	raw.glVertex2fv,
	2,
	arrays.GLfloatArray,
	'v',
)

glVertex2iv = arrays.setInputArraySizeType(
	raw.glVertex2iv,
	2,
	arrays.GLintArray,
	'v',
)

glVertex2sv = arrays.setInputArraySizeType(
	raw.glVertex2sv,
	2,
	arrays.GLshortArray,
	'v',
)

glVertex3dv = arrays.setInputArraySizeType(
	raw.glVertex3dv,
	3,
	arrays.GLdoubleArray,
	'v',
)

glVertex3fv = arrays.setInputArraySizeType(
	raw.glVertex3fv,
	3,
	arrays.GLfloatArray,
	'v',
)

glVertex3iv = arrays.setInputArraySizeType(
	raw.glVertex3iv,
	3,
	arrays.GLintArray,
	'v',
)

glVertex3sv = arrays.setInputArraySizeType(
	raw.glVertex3sv,
	3,
	arrays.GLshortArray,
	'v',
)

glVertex4dv = arrays.setInputArraySizeType(
	raw.glVertex4dv,
	4,
	arrays.GLdoubleArray,
	'v',
)

glVertex4fv = arrays.setInputArraySizeType(
	raw.glVertex4fv,
	4,
	arrays.GLfloatArray,
	'v',
)

glVertex4iv = arrays.setInputArraySizeType(
	raw.glVertex4iv,
	4,
	arrays.GLintArray,
	'v',
)

glVertex4sv = arrays.setInputArraySizeType(
	raw.glVertex4sv,
	4,
	arrays.GLshortArray,
	'v',
)

__all__ = [
	'glAreTexturesResident',
	'glBitmap',
	'glClipPlane',
	'glColor3bv',
	'glColor3dv',
	'glColor3fv',
	'glColor3iv',
	'glColor3sv',
	'glColor3ubv',
	'glColor3uiv',
	'glColor3usv',
	'glColor4bv',
	'glColor4dv',
	'glColor4fv',
	'glColor4iv',
	'glColor4sv',
	'glColor4ubv',
	'glColor4uiv',
	'glColor4usv',
	'glDeleteTextures',
	'glEvalCoord1dv',
	'glEvalCoord1fv',
	'glEvalCoord2dv',
	'glEvalCoord2fv',
	'glFeedbackBuffer',
	'glFogfv',
	'glFogiv',
	'glGenTextures',
	'glGetClipPlane',
	'glGetDoublev',
	'glGetFloatv',
	'glGetIntegerv',
	'glGetLightfv',
	'glGetLightiv',
	'glGetMapdv',
	'glGetMapfv',
	'glGetMapiv',
	'glGetMaterialfv',
	'glGetMaterialiv',
	'glGetPixelMapfv',
	'glGetPixelMapuiv',
	'glGetPixelMapusv',
	'glGetPolygonStipple',
	'glGetTexEnvfv',
	'glGetTexEnviv',
	'glGetTexGendv',
	'glGetTexGenfv',
	'glGetTexGeniv',
	'glGetTexLevelParameterfv',
	'glGetTexLevelParameteriv',
	'glGetTexParameterfv',
	'glGetTexParameteriv',
	'glIndexdv',
	'glIndexfv',
	'glIndexiv',
	'glIndexsv',
	'glIndexubv',
	'glLightModelfv',
	'glLightModeliv',
	'glLightfv',
	'glLightiv',
	'glLoadMatrixd',
	'glLoadMatrixf',
	'glMap1d',
	'glMap1f',
	'glMap2d',
	'glMap2f',
	'glMaterialfv',
	'glMaterialiv',
	'glMultMatrixd',
	'glMultMatrixf',
	'glNormal3bv',
	'glNormal3dv',
	'glNormal3fv',
	'glNormal3iv',
	'glNormal3sv',
	'glPixelMapfv',
	'glPixelMapuiv',
	'glPixelMapusv',
	'glPolygonStipple',
	'glPrioritizeTextures',
	'glRasterPos2dv',
	'glRasterPos2fv',
	'glRasterPos2iv',
	'glRasterPos2sv',
	'glRasterPos3dv',
	'glRasterPos3fv',
	'glRasterPos3iv',
	'glRasterPos3sv',
	'glRasterPos4dv',
	'glRasterPos4fv',
	'glRasterPos4iv',
	'glRasterPos4sv',
	'glRectdv',
	'glRectfv',
	'glRectiv',
	'glRectsv',
	'glSelectBuffer',
	'glTexCoord1dv',
	'glTexCoord1fv',
	'glTexCoord1iv',
	'glTexCoord1sv',
	'glTexCoord2dv',
	'glTexCoord2fv',
	'glTexCoord2iv',
	'glTexCoord2sv',
	'glTexCoord3dv',
	'glTexCoord3fv',
	'glTexCoord3iv',
	'glTexCoord3sv',
	'glTexCoord4dv',
	'glTexCoord4fv',
	'glTexCoord4iv',
	'glTexCoord4sv',
	'glTexEnvfv',
	'glTexEnviv',
	'glTexGendv',
	'glTexGenfv',
	'glTexGeniv',
	'glTexParameterfv',
	'glTexParameteriv',
	'glVertex2dv',
	'glVertex2fv',
	'glVertex2iv',
	'glVertex2sv',
	'glVertex3dv',
	'glVertex3fv',
	'glVertex3iv',
	'glVertex3sv',
	'glVertex4dv',
	'glVertex4fv',
	'glVertex4iv',
	'glVertex4sv'
]




'''OpenGL extension EXT.gpu_shader4

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/gpu_shader4.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_gpu_shader4'
GL_SAMPLER_1D_ARRAY_EXT = constant.Constant( 'GL_SAMPLER_1D_ARRAY_EXT', 0x8DC0 )
GL_SAMPLER_2D_ARRAY_EXT = constant.Constant( 'GL_SAMPLER_2D_ARRAY_EXT', 0x8DC1 )
GL_SAMPLER_BUFFER_EXT = constant.Constant( 'GL_SAMPLER_BUFFER_EXT', 0x8DC2 )
GL_SAMPLER_1D_ARRAY_SHADOW_EXT = constant.Constant( 'GL_SAMPLER_1D_ARRAY_SHADOW_EXT', 0x8DC3 )
GL_SAMPLER_2D_ARRAY_SHADOW_EXT = constant.Constant( 'GL_SAMPLER_2D_ARRAY_SHADOW_EXT', 0x8DC4 )
GL_SAMPLER_CUBE_SHADOW_EXT = constant.Constant( 'GL_SAMPLER_CUBE_SHADOW_EXT', 0x8DC5 )
GL_UNSIGNED_INT_VEC2_EXT = constant.Constant( 'GL_UNSIGNED_INT_VEC2_EXT', 0x8DC6 )
GL_UNSIGNED_INT_VEC3_EXT = constant.Constant( 'GL_UNSIGNED_INT_VEC3_EXT', 0x8DC7 )
GL_UNSIGNED_INT_VEC4_EXT = constant.Constant( 'GL_UNSIGNED_INT_VEC4_EXT', 0x8DC8 )
GL_INT_SAMPLER_1D_EXT = constant.Constant( 'GL_INT_SAMPLER_1D_EXT', 0x8DC9 )
GL_INT_SAMPLER_2D_EXT = constant.Constant( 'GL_INT_SAMPLER_2D_EXT', 0x8DCA )
GL_INT_SAMPLER_3D_EXT = constant.Constant( 'GL_INT_SAMPLER_3D_EXT', 0x8DCB )
GL_INT_SAMPLER_CUBE_EXT = constant.Constant( 'GL_INT_SAMPLER_CUBE_EXT', 0x8DCC )
GL_INT_SAMPLER_2D_RECT_EXT = constant.Constant( 'GL_INT_SAMPLER_2D_RECT_EXT', 0x8DCD )
GL_INT_SAMPLER_1D_ARRAY_EXT = constant.Constant( 'GL_INT_SAMPLER_1D_ARRAY_EXT', 0x8DCE )
GL_INT_SAMPLER_2D_ARRAY_EXT = constant.Constant( 'GL_INT_SAMPLER_2D_ARRAY_EXT', 0x8DCF )
GL_INT_SAMPLER_BUFFER_EXT = constant.Constant( 'GL_INT_SAMPLER_BUFFER_EXT', 0x8DD0 )
GL_UNSIGNED_INT_SAMPLER_1D_EXT = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_1D_EXT', 0x8DD1 )
GL_UNSIGNED_INT_SAMPLER_2D_EXT = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_2D_EXT', 0x8DD2 )
GL_UNSIGNED_INT_SAMPLER_3D_EXT = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_3D_EXT', 0x8DD3 )
GL_UNSIGNED_INT_SAMPLER_CUBE_EXT = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_CUBE_EXT', 0x8DD4 )
GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT', 0x8DD5 )
GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT', 0x8DD6 )
GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT', 0x8DD7 )
GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT', 0x8DD8 )
glGetUniformuivEXT = platform.createExtensionFunction( 
	'glGetUniformuivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, arrays.GLuintArray,),
	doc = 'glGetUniformuivEXT( GLuint(program), GLint(location), GLuintArray(params) ) -> None',
	argNames = ('program', 'location', 'params',),
)

glBindFragDataLocationEXT = platform.createExtensionFunction( 
	'glBindFragDataLocationEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, arrays.GLcharArray,),
	doc = 'glBindFragDataLocationEXT( GLuint(program), GLuint(color), GLcharArray(name) ) -> None',
	argNames = ('program', 'color', 'name',),
)

glGetFragDataLocationEXT = platform.createExtensionFunction( 
	'glGetFragDataLocationEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLint, 
	argTypes=(constants.GLuint, arrays.GLcharArray,),
	doc = 'glGetFragDataLocationEXT( GLuint(program), GLcharArray(name) ) -> constants.GLint',
	argNames = ('program', 'name',),
)

glUniform1uiEXT = platform.createExtensionFunction( 
	'glUniform1uiEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLuint,),
	doc = 'glUniform1uiEXT( GLint(location), GLuint(v0) ) -> None',
	argNames = ('location', 'v0',),
)

glUniform2uiEXT = platform.createExtensionFunction( 
	'glUniform2uiEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLuint, constants.GLuint,),
	doc = 'glUniform2uiEXT( GLint(location), GLuint(v0), GLuint(v1) ) -> None',
	argNames = ('location', 'v0', 'v1',),
)

glUniform3uiEXT = platform.createExtensionFunction( 
	'glUniform3uiEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glUniform3uiEXT( GLint(location), GLuint(v0), GLuint(v1), GLuint(v2) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2',),
)

glUniform4uiEXT = platform.createExtensionFunction( 
	'glUniform4uiEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLuint, constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glUniform4uiEXT( GLint(location), GLuint(v0), GLuint(v1), GLuint(v2), GLuint(v3) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2', 'v3',),
)

glUniform1uivEXT = platform.createExtensionFunction( 
	'glUniform1uivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLuintArray,),
	doc = 'glUniform1uivEXT( GLint(location), GLsizei(count), GLuintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform2uivEXT = platform.createExtensionFunction( 
	'glUniform2uivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLuintArray,),
	doc = 'glUniform2uivEXT( GLint(location), GLsizei(count), GLuintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform3uivEXT = platform.createExtensionFunction( 
	'glUniform3uivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLuintArray,),
	doc = 'glUniform3uivEXT( GLint(location), GLsizei(count), GLuintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform4uivEXT = platform.createExtensionFunction( 
	'glUniform4uivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLuintArray,),
	doc = 'glUniform4uivEXT( GLint(location), GLsizei(count), GLuintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)


def glInitGpuShader4EXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.shadow_funcs

Overview (from the spec)
	
	This extension generalizes the GL_ARB_shadow extension to support all
	eight binary texture comparison functions rather than just GL_LEQUAL
	and GL_GEQUAL.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/shadow_funcs.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_shadow_funcs'



def glInitShadowFuncsEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.texture_perturb_normal

Overview (from the spec)
	
	This extension defines a mechanism for using texture values to perturb
	the fragment normal vector prior to fragment lighting.  It enables a
	direct implementation of the original formulation of bump mapping by
	Blinn.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/texture_perturb_normal.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_texture_perturb_normal'
GL_PERTURB_EXT = constant.Constant( 'GL_PERTURB_EXT', 0x85AE )
GL_TEXTURE_NORMAL_EXT = constant.Constant( 'GL_TEXTURE_NORMAL_EXT', 0x85AF )
glTextureNormalEXT = platform.createExtensionFunction( 
	'glTextureNormalEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glTextureNormalEXT( GLenum(mode) ) -> None',
	argNames = ('mode',),
)


def glInitTexturePerturbNormalEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ATI.separate_stencil

Overview (from the spec)
	
	This extension provides the ability to modify the stencil buffer
	differently based on the facing direction of the primitive that
	generated the fragment.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ATI/separate_stencil.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ATI_separate_stencil'
GL_STENCIL_BACK_FUNC_ATI = constant.Constant( 'GL_STENCIL_BACK_FUNC_ATI', 0x8800 )
GL_STENCIL_BACK_FAIL_ATI = constant.Constant( 'GL_STENCIL_BACK_FAIL_ATI', 0x8801 )
GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI = constant.Constant( 'GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI', 0x8802 )
GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI = constant.Constant( 'GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI', 0x8803 )
glStencilOpSeparateATI = platform.createExtensionFunction( 
	'glStencilOpSeparateATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLenum, constants.GLenum,),
	doc = 'glStencilOpSeparateATI( GLenum(face), GLenum(sfail), GLenum(dpfail), GLenum(dppass) ) -> None',
	argNames = ('face', 'sfail', 'dpfail', 'dppass',),
)

glStencilFuncSeparateATI = platform.createExtensionFunction( 
	'glStencilFuncSeparateATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLint, constants.GLuint,),
	doc = 'glStencilFuncSeparateATI( GLenum(frontfunc), GLenum(backfunc), GLint(ref), GLuint(mask) ) -> None',
	argNames = ('frontfunc', 'backfunc', 'ref', 'mask',),
)


def glInitSeparateStencilATI():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.bgra

Overview (from the spec)
	
	EXT_bgra extends the list of host-memory color formats.
	Specifically, it provides formats which match the memory layout of
	Windows DIBs so that applications can use the same data in both
	Windows API calls and OpenGL pixel API calls.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/bgra.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_bgra'
GL_BGR_EXT = constant.Constant( 'GL_BGR_EXT', 0x80E0 )
GL_BGRA_EXT = constant.Constant( 'GL_BGRA_EXT', 0x80E1 )


def glInitBgraEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension VERSION.GL_1_5

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/VERSION/GL_1_5.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_VERSION_GL_1_5'
GL_BUFFER_SIZE = constant.Constant( 'GL_BUFFER_SIZE', 0x8764 )
GL_BUFFER_USAGE = constant.Constant( 'GL_BUFFER_USAGE', 0x8765 )
GL_QUERY_COUNTER_BITS = constant.Constant( 'GL_QUERY_COUNTER_BITS', 0x8864 )
GL_CURRENT_QUERY = constant.Constant( 'GL_CURRENT_QUERY', 0x8865 )
GL_QUERY_RESULT = constant.Constant( 'GL_QUERY_RESULT', 0x8866 )
GL_QUERY_RESULT_AVAILABLE = constant.Constant( 'GL_QUERY_RESULT_AVAILABLE', 0x8867 )
GL_ARRAY_BUFFER = constant.Constant( 'GL_ARRAY_BUFFER', 0x8892 )
GL_ELEMENT_ARRAY_BUFFER = constant.Constant( 'GL_ELEMENT_ARRAY_BUFFER', 0x8893 )
GL_ARRAY_BUFFER_BINDING = constant.Constant( 'GL_ARRAY_BUFFER_BINDING', 0x8894 )
GL_ELEMENT_ARRAY_BUFFER_BINDING = constant.Constant( 'GL_ELEMENT_ARRAY_BUFFER_BINDING', 0x8895 )
GL_VERTEX_ARRAY_BUFFER_BINDING = constant.Constant( 'GL_VERTEX_ARRAY_BUFFER_BINDING', 0x8896 )
GL_NORMAL_ARRAY_BUFFER_BINDING = constant.Constant( 'GL_NORMAL_ARRAY_BUFFER_BINDING', 0x8897 )
GL_COLOR_ARRAY_BUFFER_BINDING = constant.Constant( 'GL_COLOR_ARRAY_BUFFER_BINDING', 0x8898 )
GL_INDEX_ARRAY_BUFFER_BINDING = constant.Constant( 'GL_INDEX_ARRAY_BUFFER_BINDING', 0x8899 )
GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING = constant.Constant( 'GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING', 0x889A )
GL_EDGE_FLAG_ARRAY_BUFFER_BINDING = constant.Constant( 'GL_EDGE_FLAG_ARRAY_BUFFER_BINDING', 0x889B )
GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING', 0x889C )
GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING', 0x889D )
GL_WEIGHT_ARRAY_BUFFER_BINDING = constant.Constant( 'GL_WEIGHT_ARRAY_BUFFER_BINDING', 0x889E )
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING', 0x889F )
GL_READ_ONLY = constant.Constant( 'GL_READ_ONLY', 0x88B8 )
GL_WRITE_ONLY = constant.Constant( 'GL_WRITE_ONLY', 0x88B9 )
GL_READ_WRITE = constant.Constant( 'GL_READ_WRITE', 0x88BA )
GL_BUFFER_ACCESS = constant.Constant( 'GL_BUFFER_ACCESS', 0x88BB )
GL_BUFFER_MAPPED = constant.Constant( 'GL_BUFFER_MAPPED', 0x88BC )
GL_BUFFER_MAP_POINTER = constant.Constant( 'GL_BUFFER_MAP_POINTER', 0x88BD )
GL_STREAM_DRAW = constant.Constant( 'GL_STREAM_DRAW', 0x88E0 )
GL_STREAM_READ = constant.Constant( 'GL_STREAM_READ', 0x88E1 )
GL_STREAM_COPY = constant.Constant( 'GL_STREAM_COPY', 0x88E2 )
GL_STATIC_DRAW = constant.Constant( 'GL_STATIC_DRAW', 0x88E4 )
GL_STATIC_READ = constant.Constant( 'GL_STATIC_READ', 0x88E5 )
GL_STATIC_COPY = constant.Constant( 'GL_STATIC_COPY', 0x88E6 )
GL_DYNAMIC_DRAW = constant.Constant( 'GL_DYNAMIC_DRAW', 0x88E8 )
GL_DYNAMIC_READ = constant.Constant( 'GL_DYNAMIC_READ', 0x88E9 )
GL_DYNAMIC_COPY = constant.Constant( 'GL_DYNAMIC_COPY', 0x88EA )
GL_SAMPLES_PASSED = constant.Constant( 'GL_SAMPLES_PASSED', 0x8914 )
glGenQueries = platform.createExtensionFunction( 
	'glGenQueries', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glGenQueries( GLsizei(n), GLuintArray(ids) ) -> None',
	argNames = ('n', 'ids',),
)

glDeleteQueries = platform.createExtensionFunction( 
	'glDeleteQueries', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glDeleteQueries( GLsizei(n), GLuintArray(ids) ) -> None',
	argNames = ('n', 'ids',),
)

glIsQuery = platform.createExtensionFunction( 
	'glIsQuery', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLuint,),
	doc = 'glIsQuery( GLuint(id) ) -> constants.GLboolean',
	argNames = ('id',),
)

glBeginQuery = platform.createExtensionFunction( 
	'glBeginQuery', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint,),
	doc = 'glBeginQuery( GLenum(target), GLuint(id) ) -> None',
	argNames = ('target', 'id',),
)

glEndQuery = platform.createExtensionFunction( 
	'glEndQuery', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glEndQuery( GLenum(target) ) -> None',
	argNames = ('target',),
)

glGetQueryiv = platform.createExtensionFunction( 
	'glGetQueryiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetQueryiv( GLenum(target), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('target', 'pname', 'params',),
)

glGetQueryObjectiv = platform.createExtensionFunction( 
	'glGetQueryObjectiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetQueryObjectiv( GLuint(id), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('id', 'pname', 'params',),
)

glGetQueryObjectuiv = platform.createExtensionFunction( 
	'glGetQueryObjectuiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLuintArray,),
	doc = 'glGetQueryObjectuiv( GLuint(id), GLenum(pname), GLuintArray(params) ) -> None',
	argNames = ('id', 'pname', 'params',),
)

glBindBuffer = platform.createExtensionFunction( 
	'glBindBuffer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint,),
	doc = 'glBindBuffer( GLenum(target), GLuint(buffer) ) -> None',
	argNames = ('target', 'buffer',),
)

glDeleteBuffers = platform.createExtensionFunction( 
	'glDeleteBuffers', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glDeleteBuffers( GLsizei(n), GLuintArray(buffers) ) -> None',
	argNames = ('n', 'buffers',),
)

glGenBuffers = platform.createExtensionFunction( 
	'glGenBuffers', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glGenBuffers( GLsizei(n), GLuintArray(buffers) ) -> None',
	argNames = ('n', 'buffers',),
)

glIsBuffer = platform.createExtensionFunction( 
	'glIsBuffer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLuint,),
	doc = 'glIsBuffer( GLuint(buffer) ) -> constants.GLboolean',
	argNames = ('buffer',),
)

glBufferData = platform.createExtensionFunction( 
	'glBufferData', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLsizeiptr, ctypes.c_void_p, constants.GLenum,),
	doc = 'glBufferData( GLenum(target), GLsizeiptr(size), c_void_p(data), GLenum(usage) ) -> None',
	argNames = ('target', 'size', 'data', 'usage',),
)

glBufferSubData = platform.createExtensionFunction( 
	'glBufferSubData', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLintptr, constants.GLsizeiptr, ctypes.c_void_p,),
	doc = 'glBufferSubData( GLenum(target), GLintptr(offset), GLsizeiptr(size), c_void_p(data) ) -> None',
	argNames = ('target', 'offset', 'size', 'data',),
)

glGetBufferSubData = platform.createExtensionFunction( 
	'glGetBufferSubData', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLintptr, constants.GLsizeiptr, ctypes.c_void_p,),
	doc = 'glGetBufferSubData( GLenum(target), GLintptr(offset), GLsizeiptr(size), c_void_p(data) ) -> None',
	argNames = ('target', 'offset', 'size', 'data',),
)

glMapBuffer = platform.createExtensionFunction( 
	'glMapBuffer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=ctypes.c_void_p, 
	argTypes=(constants.GLenum, constants.GLenum,),
	doc = 'glMapBuffer( GLenum(target), GLenum(access) ) -> ctypes.c_void_p',
	argNames = ('target', 'access',),
)

glUnmapBuffer = platform.createExtensionFunction( 
	'glUnmapBuffer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLenum,),
	doc = 'glUnmapBuffer( GLenum(target) ) -> constants.GLboolean',
	argNames = ('target',),
)

glGetBufferParameteriv = platform.createExtensionFunction( 
	'glGetBufferParameteriv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetBufferParameteriv( GLenum(target), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('target', 'pname', 'params',),
)

glGetBufferPointerv = platform.createExtensionFunction( 
	'glGetBufferPointerv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, ctypes.POINTER(ctypes.c_void_p),),
	doc = 'glGetBufferPointerv( GLenum(target), GLenum(pname), POINTER(ctypes.c_void_p)(params) ) -> None',
	argNames = ('target', 'pname', 'params',),
)




'''OpenGL extension ARB.draw_instanced

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/draw_instanced.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_draw_instanced'

glDrawArraysInstancedARB = platform.createExtensionFunction( 
	'glDrawArraysInstancedARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint, constants.GLsizei, constants.GLsizei,),
	doc = 'glDrawArraysInstancedARB( GLenum(mode), GLint(first), GLsizei(count), GLsizei(primcount) ) -> None',
	argNames = ('mode', 'first', 'count', 'primcount',),
)

glDrawElementsInstancedARB = platform.createExtensionFunction( 
	'glDrawElementsInstancedARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLsizei, constants.GLenum, ctypes.c_void_p, constants.GLsizei,),
	doc = 'glDrawElementsInstancedARB( GLenum(mode), GLsizei(count), GLenum(type), c_void_p(indices), GLsizei(primcount) ) -> None',
	argNames = ('mode', 'count', 'type', 'indices', 'primcount',),
)


def glInitDrawInstancedARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )
